@@ -23,7 +23,7 @@ double Graph::Dist2(int *i, int *j)
 }
 /********************************************************************************************/
 /********************************************************************************************/
-void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDists0, int *toroidal0, int *dbg0)
+void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDists0, double *preDists, int *toroidal0, int *dbg0)
 {
 	if(*dbg0)printf("intializing graph-object... ");
 
@@ -37,7 +37,15 @@ void Graph::Init(Pp *pp0, int *gtype0, double *par0, double *prepR0, int *doDist
 	pdists = &distTriangle;
 	gtype = gtype0;
 	Dist = &Graph::Dist1;  // calculate anew every time
-	if(*doDists)  // distance triangle
+
+	if(preDists[0]>=0)
+	{
+		if(*dbg)printf("Setting precalculated distances...");
+		setDists(pp, pdists, preDists);
+		Dist = &Graph::Dist2;
+		if(*dbg)printf("ok. ");
+	}
+	else if(*doDists)  // distance triangle
 	{
 		if(*dbg)printf("Precalculating distances...");
 		calcDists(pp, pdists,toroidal);
@@ -63,7 +71,6 @@ SEXP Graph::toSEXP()
 //transform a std::vector<std::vector<int> > to SEXP, desctructive
 {
 	SEXP graph, *node;
-
 	PROTECT(graph = allocVector(VECSXP, this->nodelist.size()));
 	int i,j, *p, n;
 	for(i=0;i< (int) this->nodelist.size();i++)
@@ -98,7 +105,7 @@ void Graph::remove_duplicates()
 		for(j=0; j < (int) this->nodelist.at(i).size() ; j++)
 		{
 			isnew=1;
-			for(k=0;k<node->size();k++)
+			for(k=0;k<(int)node->size();k++)
 			{
 				if(node->at(k) == this->nodelist.at(i).at(j))
 				{
@@ -165,6 +172,7 @@ void Graph::sg_calc()
 	else if(*gtype==8) this->sg_RST();
 	else if(*gtype==9) this->sg_RNG();
 	else if(*gtype==10) this->sg_CCC();
+	else if(*gtype==11) this->sg_STIR();
 }
 
 /********************************************************************************************/
@@ -207,7 +215,7 @@ void Graph::sg_shrink_geometric(double *R)
 				node->push_back(j0+1);
 		}
 		nodelist[i].clear();nodelist[i].resize(0);
-		for (j = 0; j < node->size(); ++j) this->nodelist[i].push_back(node->at(j));
+		for (j = 0; j < (int)node->size(); ++j) this->nodelist[i].push_back(node->at(j));
 		delete node;
 	}
 	if(*dbg)printf(" Ok.");
@@ -299,6 +307,8 @@ void Graph::sg_shrink_knn()
 /********************************************************************************************/
 void Graph::sg_gabriel()
 {
+	int kk = (int) par[0];
+	if(*dbg & kk>0)printf("%i-",kk);
 	if(*dbg)printf("Gabriel:");
 	int i,j,k, empty,m,l,h;
 	double x0,y0,R2, d;
@@ -313,7 +323,7 @@ void Graph::sg_gabriel()
 			  y0 = fabs(pp->y[i]-pp->y[j])/2.0+fmin(pp->y[i],pp->y[j]);
 			  R2 = ( pow(pp->x[i]-pp->x[j],2) + pow(pp->y[i]-pp->y[j],2) )/4.0;
 			  //		brute force
-			  empty = 1;
+			  empty = 1+kk;
 			  for(k=0;k<*pp->n;k++)
 			  {
 				  if(k != i)
@@ -322,8 +332,8 @@ void Graph::sg_gabriel()
 						  d = pow(x0-pp->x[k],2) + pow(y0-pp->y[k],2);
 						  if( d<R2 )
 						  {
-							  empty = 0;
-							  break;
+							  empty = empty - 1;
+							  if(empty == 0) break;
 						  }
 					  }
 			  }
@@ -344,8 +354,8 @@ void Graph::sg_gabriel()
 				x0 = fabs(this->pp->x[i]-this->pp->x[j])/2.0+fmin(this->pp->x[i],this->pp->x[j]);
 				y0 = fabs(this->pp->y[i]-this->pp->y[j])/2.0+fmin(this->pp->y[i],this->pp->y[j]);
 				R2 = (pow(this->pp->x[i]-this->pp->x[j],2) + pow(this->pp->y[i]-this->pp->y[j],2) )/4.0;
-				empty = 1;
-				for(m=0; m < this->nodelist[i].size();m++) // the small ball is included in the preprocessing ball
+				empty = 1+kk;
+				for(m=0; m <(int) this->nodelist[i].size();m++) // the small ball is included in the preprocessing ball
 				{
 					k =  (int) this->nodelist[i][m]-1;
 					if(k != i)
@@ -354,8 +364,8 @@ void Graph::sg_gabriel()
 							d = pow(x0-pp->x[k],2) + pow(y0-pp->y[k],2);
 							if( d<R2 )
 							{
-								empty = 0;
-								break;
+								empty = empty - 1;
+								if(empty == 0) break;
 							}
 						}
 				}
@@ -365,7 +375,7 @@ void Graph::sg_gabriel()
 				}
 			}
 			nodelist[i].clear();nodelist[i].resize(0);
-			for(h=0;h<node->size();h++) nodelist[i].push_back(node->at(h));
+			for(h=0;h<(int)node->size();h++) nodelist[i].push_back(node->at(h));
 			delete node;
 		}
 	}
@@ -400,10 +410,10 @@ void Graph::sg_delauney()
 		for(i = 0 ; i< *pp->n ;  i++)
 		{
 			node = new std::vector<int>;
-			for( l=0 ; l < nodelist[i].size()-1; l++ )
+			for( l=0 ; l < (int)nodelist[i].size()-1; l++ )
 			{
 				j = nodelist[i][l]-1;
-				for(h=l+1; h < nodelist[i].size();h++ )
+				for(h=l+1; h < (int)nodelist[i].size();h++ )
 				{
 					k = nodelist[i][h]-1;
 					if( Empty(pp->x,pp->y,pp->n,i,j,k,dummy, dummy, dummy) )
@@ -414,7 +424,7 @@ void Graph::sg_delauney()
 				}
 			}
 			nodelist[i].clear();nodelist[i].resize(0);
-			for(h=0;h<node->size();h++) nodelist[i].push_back((*node).at(h));
+			for(h=0;h<(int)node->size();h++) nodelist[i].push_back((*node).at(h));
 			delete node;
 		}
 		this->remove_duplicates();
@@ -432,10 +442,8 @@ void Graph::sg_MST()
   done[0] = 0;
   dn = 1;
   int left=*this->pp->n-dn;
-//   double sum;
   while( left > 0 )
   {
-    //if(*print_sg) if((left+1)%100==0) printf("MST: %i/%i         \r",left,*n);
     apu2 = MAX_DOUBLE;
     for(i=1; i<*this->pp->n;i++){
       zz = 1;
@@ -445,7 +453,7 @@ void Graph::sg_MST()
           zz=0;
           break;
         }
-        apu1 = (this->*Dist)(&i,&done[j]); //dists[i*(*n)+done[j]];
+        apu1 = (this->*Dist)(&i,&done[j]);
         if( apu1<apu0 ){
           apu0=apu1;
           k0=i;
@@ -460,13 +468,10 @@ void Graph::sg_MST()
         }
       }
     }
-//     sum+=apu2;
     done[dn] = k;
     dn++;
     left--;
     this->nodelist[l].push_back(k+1);
-    //e[l*(*n)+k] = 1;
-//     printf("\r");
   }
   if(*this->dbg)printf(" Ok.");
 }
@@ -586,6 +591,43 @@ void Graph::sg_CCC()
 							addNew(i,j+1);
 	if(*dbg) printf(" Ok.");
 }
+/********************************************************************************************/
+double Attenuate(double r, double alpha)
+{
+	return pow(1.0+r,-alpha);
+}
+
+void Graph::sg_STIR()
+{
+	if(*dbg) printf("Signal-To-Noise-Ratio graph, noise=%f,alpha=%f,beta=%f,gamma=%f: ",par[0],par[1],par[2],par[3]);
+	int i,j;
+	double noise0 = par[0], alpha=par[1],beta=par[2],gamma=par[3], noise[*pp->n], sij,sji,s;
+	//first we compute the interference for each location including the sending transmitter
+	for(i=0;i<*pp->n;i++)
+	{
+		noise[i]=0.0;
+		for(j=0;j<*pp->n;j++)
+			if(i!=j) noise[i] = noise[i]+ pp->mass[j]*Attenuate((this->*Dist)(&i,&j),alpha);
+	}
+
+	for(i=0;i<(*pp->n-1);i++)
+		for(j=i+1;j<*pp->n;j++)
+		{
+			sij = pp->mass[i]*Attenuate((this->*Dist)(&i,&j),alpha);
+			sji = pp->mass[j]*Attenuate((this->*Dist)(&i,&j),alpha);
+			sij = sij/(noise0 + gamma* (noise[j]-sij));
+			sij = sji/(noise0 + gamma* (noise[i]-sji));
+			s = fminf(sij,sji);
+			if(s >= beta )
+			{
+				addNew(i,j+1);
+				addNew(j,i+1);
+			}
+		}
+	if(*dbg) printf(" Ok.");
+}
+
+
 /********************************************************************************************/
 // cut (remove) the graph edges longer than R
 void Graph::sg_cut(double *R)
@@ -615,7 +657,7 @@ void Graph::sg_cut(double *R)
 
 void Graph::sg_prune(double *lev)
 {
-	int level = (int) *lev , i, leaf, count=0, notnew, prev, next;
+	int level = (int) *lev , i, leaf, count=0, prev, next;
 	std::vector<int> left;
 	std::vector<int> branch, *pnode;
 	left.resize(0);
@@ -657,10 +699,10 @@ void Graph::sg_prune(double *lev)
 		{
 			pnode = new std::vector<int>;
 			pnode->resize(0);
-			for(i=0; i < branch.size();i++)
+			for(i=0; i < (int)branch.size();i++)
 				nodelist.at(branch.at(i)-1).clear();
 
-			for(i=0; i < nodelist.at(next-1).size();i++)
+			for(i=0; i < (int)nodelist.at(next-1).size();i++)
 				if(nodelist.at(next-1).at(i) != prev)
 					pnode->push_back(nodelist.at(next-1).at(i));
 			nodelist.at(next-1).swap(*pnode);
@@ -674,4 +716,7 @@ void Graph::sg_prune(double *lev)
 	if(*dbg)printf(" Ok (%i branches pruned).",count);
 }
 /********************************************************************************************/
+
+
+
 // EOF
